/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.engine;

// EXTERNAL IMPORTS
import javax.microedition.broadcast.esg.Query;
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.Attribute;
import javax.microedition.broadcast.esg.ProgramEvent;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.esg.ServiceGuide;
import javax.microedition.broadcast.esg.QueryException;
import javax.microedition.broadcast.esg.ServiceGuideListener;
import java.util.Vector;
import java.util.Date;

/**
 * Controls the electronic service guide
 */
public class ServiceGuideController {

    public static final int QUERY_SERVICES = 0;
    public static final int QUERY_PROGRAMS = 1;
    // Singleton instance
    private static ServiceGuideController serviceGuideController;
    // Current Service guide
    private ServiceGuide serviceGuide;
    // Current query
    private Query query;
    // Query type
    private int queryType;
    // Sorting keys
    private Vector sortKeys = new Vector();
    // Currently selected service
    private Service selectedService;
    // Query result
    private Vector queryResult;

    /**
     * Returns the sigleton instance of this class
     */
    public static ServiceGuideController getInstance() {
        if (serviceGuideController == null) {
            serviceGuideController = new ServiceGuideController();
        }

        return serviceGuideController;
    }

    /**
     * Initializes the service guide controller
     */
    public synchronized void initialize() {
        if (serviceGuide == null) {
            // Get first service guide
            serviceGuide = ServiceGuide.getAllServiceGuides()[ 0];

            query = null;
            System.out.println("ServiceGuideController.initialize: size = " +
                    sortKeys.size());
        }
    }

    /**
     * Deinitializes the service guide controller
     */
    public synchronized void deinitialize() {
        serviceGuide = null;
    }

    /**
     * Sets the executed query
     *
     * @param argQuery The query to be executed
     * @param aSortKeys Keys which are used in sorting the result
     * @param argQueryType The type of the query
     */
    public void setQuery(Query argQuery, int argQueryType) {
        queryType = argQueryType;
        query = argQuery;
    }

    /**
     * Returns the current query. Null is returned if no query has been set
     */
    public Query getQuery() {
        return query;
    }

    /**
     * Selects a service
     */
    public void setService(Service argService) {
        selectedService = argService;
    }

    /**
     * Gets the selected service
     */
    public Service getService() {
        return selectedService;
    }

    /**
     * Sets the keys used to sort the query results
     *
     * @param argSortKeys Keys which are used in sorting the result
     */
    public void setSortKeys(Vector argSortKeys) {
        if (argSortKeys != null) {
            sortKeys = argSortKeys;
            System.out.println("ServiceGuideController.setSortKeys: size = " +
                    sortKeys.size());
        }
    }

    /**
     * Resets query result sorting keys
     */
    public void resetSortKeys() {
        sortKeys = new Vector();
        System.out.println("ServiceGuideController.resetSortKeys: size = " +
                sortKeys.size());
    }

    /**
     * Executes the specified query
     */
    public Vector executeQuery() {
        checkValidity();

        try {
            int varSize = sortKeys.size();
            System.out.println("ServiceGuideController.executeQuery: size = " +
                    varSize);
            Attribute[] varSortAttribs = null;
            if (varSize > 0) {
                varSortAttribs = new Attribute[varSize];
                sortKeys.copyInto(varSortAttribs);
            }

            if (queryType == QUERY_SERVICES) {
                // Query may be null which indicates that
                // all services are returned
                Service[] varQueryResult =
                        serviceGuide.findServices(query,
                                    varSortAttribs, -1, -1);
                int varCount = varQueryResult.length;
                queryResult = new Vector(varCount);
                // Copy query result
                for (int varIndex = 0;
                        varIndex < varQueryResult.length; varIndex++) {
                    queryResult.addElement(varQueryResult[varIndex]);
                }
            } else if (queryType == QUERY_PROGRAMS) {
                // Query may be null which indicates that
                // all programs are returned
                ProgramEvent[] varQueryResult =
                        serviceGuide.findPrograms(query,
                                varSortAttribs, -1, -1);
                int varCount = varQueryResult.length;
                queryResult = new Vector(varCount);
                // Copy query result
                for (int varIndex = 0;
                        varIndex < varQueryResult.length; varIndex++) {
                    queryResult.addElement(varQueryResult[varIndex]);
                }
            }
        } catch (QueryException e) {
            throw new RuntimeException("QueryException: " + e.getMessage());
        }

        return queryResult;
    }

    /**
     * Updates the service guide
     */
    public void updateServiceGuide() {
        checkValidity();
        serviceGuide.forceUpdate();
    }

    /**
     * Adds a new listener to the current service guide
     *
     * @param argListener The listener to be registered
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    public void addListener(ServiceGuideListener argListener) {
        checkValidity();
        try {
            serviceGuide.addListener(argListener, null);
        } catch (QueryException e) {
            // Ignore at this point
        }
    }

    /**
     * Removes an existing listener from the current service guide
     *
     * @param argListener The listener to be removed
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    public void removeListener(ServiceGuideListener argListener) {
        checkValidity();
        serviceGuide.removeListener(argListener);
    }

    /**
     * Gets the last update time for the associated service guide
     *
     * @return The last updated time of the associated service guide
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    public Date getUpdateTime() {
        checkValidity();
        return serviceGuide.getLastUpdatedTime();
    }

    /**
     * Checks whether this service guide controller has been initialized or not
     *
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    private void checkValidity() {
        if (serviceGuide == null) {
            throw new IllegalStateException("Service guide not initialized");
        }
    }

    /**
     * Constructor
     */
    private ServiceGuideController() {
    }
}

// End of File

