/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// EXTERNAL IMPORTS
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.CommandListener;

/**
 * Handles the view activation and command dispatching
 * in the application
 */
public class ViewHandler implements CommandListener
{
    // Public view identifiers
    public static final int MAIN_VIEW_ID               = 0;
    public static final int PLATFORM_VIEW_ID           = 1;
    public static final int PLATFORM_PROVIDER_VIEW_ID  = 2;
    public static final int SERVICES_VIEW_ID           = 3;
    public static final int PROGRAMS_VIEW_ID           = 4;
    public static final int QUERY_COMPOSER_VIEW_ID     = 5;
    public static final int VIDEO_FORM_VIEW_ID         = 6;
    public static final int VIDEO_CANVAS_VIEW_ID       = 7;
    public static final int SORT_ATTRIBUTES_VIEW_ID    = 8;
    public static final int SERVICE_COMPONENTS_VIEW_ID = 9;
    public static final int PURCHASE_OBJECTS_VIEW_ID   = 10;
    public static final int SUBSCRIPTION_VIEW_ID       = 11;
    public static final int PRICE_OPTION_VIEW_ID       = 12;
    public static final int SUBSCRIBED_OBJECTS_VIEW_ID = 13;

    // Vector containing views
    private Hashtable views;
    private ViewBase currentView;
    // Previous views are added to the vector when the user selects
    // a new view. The views are removed when the user goes back
    // to the previous views.
    private Vector previousViews;
    private Display display;
    private MIDlet midlet;

    // Singleton instance of this view handler
    private static ViewHandler viewHandler;

    public static ViewHandler getInstance()
    {
        if (viewHandler == null)
        {
            viewHandler = new ViewHandler();
        }
        return viewHandler;
    }

    /**
     * Constructor
     */
    private ViewHandler()
    {
        views = new Hashtable();
        previousViews = new Vector(5);

        // Create all view used in the application
        views.put(
                new Integer(MAIN_VIEW_ID), new MainView());
        views.put(
                new Integer(PLATFORM_VIEW_ID), new PlatformView());
        views.put(
                new Integer(PLATFORM_PROVIDER_VIEW_ID),
                new PlatformProviderView());
        views.put(
                new Integer(PROGRAMS_VIEW_ID), new ProgramEventsView());
        views.put(
                new Integer(SERVICES_VIEW_ID), new ServicesView());
        views.put(
                new Integer(QUERY_COMPOSER_VIEW_ID), new QueryComposerView());
        views.put(
                new Integer(VIDEO_FORM_VIEW_ID), new VideoFormView());
        views.put(
                new Integer(VIDEO_CANVAS_VIEW_ID), new VideoCanvasView());
        views.put(
                new Integer(SORT_ATTRIBUTES_VIEW_ID),
                new SortAttributesView());
        views.put(
                new Integer(SERVICE_COMPONENTS_VIEW_ID),
                new ServiceComponentsView());
        views.put(
                new Integer(PURCHASE_OBJECTS_VIEW_ID),
                new PurchaseObjectsView());
        views.put(
                new Integer(SUBSCRIPTION_VIEW_ID), new SubscriptionView());
        views.put(
                new Integer(PRICE_OPTION_VIEW_ID), new PriceOptionView());
        views.put(
                new Integer(SUBSCRIBED_OBJECTS_VIEW_ID),
                new SubscribedObjectsView());
    }

    /**
     * Sets the display of the MIDlet to this view handler
     * @param argDisplay The display of the MIDlet
     */
    public void setDisplay(Display argDisplay)
    {
        display = argDisplay;
    }

    /**
     * Sets the current MIDlet to this view handler
     * The MIDlet instance is needed to exit from
     * the application
     * @param argMIDlet Current running MIDlet
     */
    public void setMIDlet(MIDlet argMIDlet)
    {
        midlet = argMIDlet;
    }

    /**
     * Switches view to the requested view
     * @param argViewId View id which is to be activated
     */
    public static void activateView(int argViewId)
    {
        ViewHandler varHandler = ViewHandler.getInstance();
        ViewBase varView =
                (ViewBase)varHandler.views.get(new Integer(argViewId));
        varHandler.doSwitchView(varView);
    }

    /**
     * Switches view to the pervious view
     * @param aViewId View id which is to be activated
     */
    public static void activatePreviousView()
    {
        ViewHandler varHandler = ViewHandler.getInstance();
        // Activate previous view (=null)
        varHandler.doSwitchView(null);
    }

    /**
     * Returns the display to which the application draws
     * @return The display to which the application draws
     */
    public static Display getDisplay()
    {
        return getInstance().display;
    }

    /**
     * Returns the current view.
     */
    public static ViewBase getCurrentView()
    {
        return getInstance().currentView;
    }

    /**
     * commandAction from CommandListener
     */
    public void commandAction(Command argCommand, Displayable argDisp)
    {
        if (argCommand.getCommandType() == Command.EXIT)
        {
            midlet.notifyDestroyed();
        }
        else
        {
            // Catch exceptions here
            try
            {
                currentView.handleCommand(argCommand, argDisp);
            }
            catch (Exception e)
            {
                e.printStackTrace();
                showAlert(e);
            }
        }
    }

    /**
     * Switches to the specified view.
     *
     * @param argView View to be activated or null if the previous
     *        view should be selected.
     */
    private void doSwitchView(ViewBase argView)
    {
        // Deactivate the current view.
        if (currentView != null)
        {
            currentView.deactivate();
        }

        // null view means that the previous view should be activated.
        if (argView == null)
        {
            if (previousViews.size() > 0)
            {
                currentView = (ViewBase)previousViews.lastElement();
                previousViews.removeElement(currentView);
            }
        }
        else
        {
            if (currentView != null)
            {
                previousViews.addElement(currentView);
            }

            currentView = argView;
        }

        // Switch view if it is available
        if (currentView != null)
        {
            currentView.setCommandListener(this);
            currentView.activate();

            display.setCurrent(currentView.getDisplayable());
        }
    }

    /**
     * Shows alert note
     */
    private void showAlert(Exception argException)
    {
        AlertNote varAlert = new AlertNote("Exception",
                                        argException.toString(),
                                        AlertType.ERROR,
                                        currentView.getDisplayable());

        varAlert.execute();
    }
}

// End of File
