/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
 */
// PACKAGE
package com.nokia.example.engine;

// EXTERNAL IMPORTS
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.ServiceContext;
import javax.microedition.broadcast.ServiceContextListener;
import javax.microedition.media.Player;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Controls the presentation of a broadcast service
 */
public class PresentationController implements ServiceContextListener {
    // Singleton instance

    private static PresentationController presentationController;
    // Default service context
    private ServiceContext serviceContext;
    // Selected service
    private Service selectedService;
    // Displayable for the video
    private Player[] currentPlayers;
    // Vector of listeners
    private Vector listeners;

    /**
     * From ServiceContextListener
     */
    public void contextUpdate(
            ServiceContext argContext,
            String argEvent,
            Object argData) {
        System.out.println(
                "PresentationController.contextUpdate() - event: " + argEvent);
        System.out.println(
                "PresentationController.contextUpdate() - aData: " + argData);

        // Use equalsIgnoreCase because RI has a bug...
        if (argEvent.equalsIgnoreCase(
                ServiceContextListener.PLAYERS_REALIZED)) {
            // Presentation has been started but the controller does not
            // start showing the content. The client must obtain the players
            // and modify the VideoControl to display the content
            currentPlayers = (Player[]) argData;
            // Presentation will start soon
            notifyListeners(PresentationListener.EVENT_PRESENTING, null);
        }
    }

    /**
     * Returns the sigleton instance of this class
     */
    public static PresentationController getInstance() {
        if (presentationController == null) {
            presentationController = new PresentationController();
        }

        return presentationController;
    }

    /**
     * Adds a new presentation listener to this object
     *
     * @param argListener The listener to be added
     */
    public void addListener(PresentationListener argListener) {
        if (!listeners.contains(argListener)) {
            listeners.addElement(argListener);
        }
    }

    /**
     * Removes an existing presentation listener from this object
     *
     * @param argListener The listener to be removed
     */
    public void removeListener(PresentationListener argListener) {
        listeners.removeElement(argListener);
    }

    /**
     * Returns a list of players which are presenting the current service
     *
     * @param argComponent The service component which player is requested
     * @return The player persenting the specified service component
     */
    public Player getPlayer(ServiceComponent argComponent) {
        checkPrepared();

        Player varPlayer = null;

        try {
            serviceContext.getPlayer(argComponent);
        } catch (Exception e) {
            System.out.println("PresentationController.getPlayer() - error: "
                    + e.toString());
        }

        return varPlayer;
    }

    /**
     * Returns a list of current players presenting the media
     */
    public Player[] getCurrentPlayers() {
        checkPrepared();

        return currentPlayers;
    }

    /**
     * Returns the default player
     */
    public Player getDefaultPlayer() {
        checkPrepared();

        return currentPlayers[ 0];
    }

    /**
     * Returns the list of selected service components
     *
     * @return The list of selected service component
     */
    public ServiceComponent[] getSelectedComponents() {
        checkPrepared();

        return serviceContext.getSelectedComponents();
    }

    /**
     * Returns the list of all service components
     *
     * @return The list of all service component
     */
    public ServiceComponent[] getAllComponents() {
        checkPrepared();
        return serviceContext.getAllComponents();
    }

    /**
     * Prepares the presentation controller. This function does not start
     * the presentation yet
     *
     * @param argService Service which will be presented when the presentation
     *        is started
     */
    public void prepare(Service argService) {
        // Get first service guide
        serviceContext = ServiceContext.getDefaultContext();
        serviceContext.addListener(this);
        selectedService = argService;
    }

    /**
     * Starts the presentation of the selected broadcast service
     *
     * @param aDisplayable The component which is used to display the service
     */
    public void start() {
        checkPrepared();

        try {
            // Select the service
            serviceContext.select(selectedService);
        } catch (Exception e) {
            throw new IllegalStateException(
                    "Presentation controller not prepared");
        }
    }

    /**
     * Stops the presentation of the selected service. Nothing happens if the
     * Service controller has not been prepared or is in closed state
     */
    public void stop() {
        if (serviceContext != null
                && serviceContext.getState() != ServiceContext.STOPPED) {
            serviceContext.stop();
        }
    }

    /**
     * Returns the currently selected service
     */
    public Service getCurrentService() {
        return selectedService;
    }

    /**
     * Returns the signal quality of this service context
     */
    public int getSignalQuality() {
        checkPrepared();

        return serviceContext.getSignalQuality();
    }

    /**
     * Notifies all registered listeners
     *
     * @param argEventType Type of the event to be delivered to the listeners
     * @param argMessage Message which will be delivered to the listeners
     */
    private void notifyListeners(int argEventType, String argMessage) {
        Enumeration varListeners = listeners.elements();
        while (varListeners.hasMoreElements()) {
            PresentationListener varListener =
                    (PresentationListener) varListeners.nextElement();
            varListener.notifyPresentationEvent(argEventType, argMessage);
        }
    }

    /**
     * Checks if this presentation controller object is prepared or not
     */
    private void checkPrepared() {
        if (serviceContext == null) {
            throw new IllegalStateException(
                    "Presentation controller not prepared");
        }
    }

    /**
     * Constructor
     */
    private PresentationController() {
        listeners = new Vector();
    }
}
// End of File

