/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Gauge;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemCommandListener;
import javax.microedition.broadcast.platform.PlatformProvider;
import javax.microedition.broadcast.platform.PlatformProviderSelector;
import javax.microedition.broadcast.platform.PlatformProviderSelectorListener;
import java.util.Vector;

/**
 * Platform view
 *
 * From this view it is possible to discover and select
 * platform providers
 */
public class PlatformView extends Form implements ViewBase,
        PlatformProviderSelectorListener, CommandListener, ItemCommandListener
{
    // Commands
    private Command cmdExit;
    private Command cmdInit;
    private Command cmdAbort;
    private Command cmdSelect;
    private Command cmdBack;
    private Command cmdUseDefault;
    // Currently discovered platform providers
    private Vector providers;
    // Progress of the discovery
    private Gauge progress;
    private AlertNote progressNote;
    // Choice group holding the found platform providers
    private ChoiceGroup choiceGroup;

    /**
     * Constructor
     */
    PlatformView()
    {
        super("Platforms");

        cmdInit = new Command("Discover", Command.SCREEN, 2);
        cmdSelect = new Command("Select", Command.SCREEN, 3);
        cmdBack = new Command("Back", Command.BACK, 2);
        cmdUseDefault = new Command("Use default", Command.SCREEN, 1);

        choiceGroup = new ChoiceGroup("Providers", ChoiceGroup.EXCLUSIVE);
        choiceGroup.setDefaultCommand(cmdSelect);
        choiceGroup.setItemCommandListener(this);

        // Create progress note
        cmdAbort = new Command("Cancel", Command.STOP, 3);
        progress = new Gauge(null, false, 100, 0);

        progressNote = new AlertNote("Discovering",
                                      "Scanning platform providers...",
                                      AlertType.INFO,
                                      this);

        progressNote.setIndicator(progress);
        progressNote.addCommand(cmdAbort);
        progressNote.setCommandListener(this);

        providers = new Vector();

        append(choiceGroup);

        addCommand(cmdInit);
        addCommand(cmdBack);
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        updateProviders();
        PlatformProvider varProvider =
                PlatformProviderSelector.getCurrentProvider();
        if (varProvider != null)
        {
            addCommand(cmdUseDefault);
        }
        // Register this object to listen platform
        // provider selector notifications
        PlatformProviderSelector.addListener(this);
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        PlatformProviderSelector.removeListener(this);
        // Do not remove platform information
        removeCommand(cmdUseDefault);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener)
    {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdInit)
        {
            // Remove any existing data from the providers lists
            progress.setValue(0);
            providers.removeAllElements();
            choiceGroup.deleteAll();
            // Start asynchronous discover process
            PlatformProviderSelector.discover();
            progressNote.execute();
        }
        else if (argCommand == cmdUseDefault)
        {
            PlatformProvider varProvider =
                    PlatformProviderSelector.getCurrentProvider();
            if (varProvider == null)
            {
                AlertNote varAlert = new AlertNote("Warning",
                                            "Platform not currently selected",
                                            AlertType.WARNING ,
                                            this);
                varAlert.execute();
            }
            else
            {
                ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
            }
        }
        else if (argCommand == cmdBack)
        {
            ViewHandler.activateView(ViewHandler.MAIN_VIEW_ID);
        }
    }

    /**
     * From ItemCommandListener.
     */
    public void commandAction(Command argCommand, Item argItem)
    {
        if (argCommand == cmdSelect)
        {
            ServiceGuideController.getInstance().deinitialize();
            handleSelectProvider();
        }
    }

    /**
     * Handles commands by internal view components of this view object.
     *
     * These commands are provided by the associated progress note.
     *
     * @param argCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void commandAction(Command argCommand, Displayable argDisplay)
    {
        if (argCommand == cmdAbort)
        {
            ViewHandler.getDisplay().setCurrent(this);
            PlatformProviderSelector.abort();
        }
    }

    /**
     * From PlatformProviderSelectorListener
     */
    public void update(int argProgress, int argFound, boolean argContinuing)
    {
        progress.setValue(argProgress);
        updateProviders();

        // Remove gauge
        if (argProgress >= 100)
        {
            ViewHandler.getDisplay().setCurrent(this);
        }
    }

    /**
     * Handles select provider command
     */
    private void handleSelectProvider()
    {
        Thread varSelectThread = new Thread()
        {
            public void run()
            {
                int varIndex = choiceGroup.getSelectedIndex();
                if (varIndex != -1)
                {
                    PlatformProvider varProvider =
                            (PlatformProvider)providers.elementAt(varIndex);

                    // Start selection progress
                    AlertNote varProgressNote = new AlertNote(
                        "Selecting",
                        "Selecting platform: " + varProvider.toString(),
                        AlertType.INFO,
                        getDisplayable());

                    Gauge varProgress = 
                            new Gauge(null, false,
                                Gauge.INDEFINITE, Gauge.CONTINUOUS_RUNNING);
                    varProgressNote.setIndicator(varProgress);
                    varProgressNote.addCommand(
                            new Command("Hide", Command.STOP, 3));
                    varProgressNote.execute();

                    Exception varException = null;
                    try
                    {
                        // Select provider
                        PlatformProviderSelector.select(varProvider);
                    }
                    catch (Exception e)
                    {
                        varException = e;
                    }

                    // Dismiss progress note
                    ViewHandler.getDisplay().setCurrent(getDisplayable());

                    if (varException != null)
                    {
                        AlertNote varAlert = new AlertNote(
                            "Error",
                            "Cannot select provider: " +
                                varException.getMessage(),
                            AlertType.ERROR,
                            getDisplayable());
                        varAlert.execute();
                    }
                    else
                    {
                        ViewHandler.activateView(
                                ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
                    }
                }
            }
        };
        varSelectThread.start();
    }

    /**
     * Updates the list of available providers
     */
    private void updateProviders()
    {
        PlatformProvider[] varProviders =
            PlatformProviderSelector.getAvailableProviders();

        for (int varIndex = 0; varIndex < varProviders.length; varIndex++)
        {
            if (!providers.contains(varProviders[ varIndex ]))
            {
                providers.addElement(varProviders[ varIndex ]);
                choiceGroup.append(varProviders[ varIndex ].toString(), null);
            }
        }
    }
}

// End of File
