/**
 * Copyright  2010 Nokia Corporation. All rights reserved. Nokia and Nokia    
 * Connecting People are registered trademarks of Nokia Corporation. Java 
 * and all Java-based marks are trademarks or registered trademarks of 
 * Sun Microsystems, Inc. Other product and company names mentioned herein 
 * may be trademarks or trade names of their respective owners. Subject to the 
 * conditions below, you may, without charge: 
 *   Use, copy, modify and/or merge copies of this software and associated 
 *    documentation files (the Software) 
 *   Publish, distribute, sub-license and/or sell new software derived from or
 *    incorporating the Software. 
 * 
 * This license, unmodified, shall be included with all copies or substantial 
 * portions of the Software that are distributed in source code form. 
 * 
 * The Software cannot constitute the primary value of any new software derived 
 * from or incorporating the Software.Any person dealing with the Software 
 * shall not misrepresent the source of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE. 
*/

// PACKAGE
package com.nokia.example.ui;

// INTERNAL IMPORTS
import com.nokia.example.engine.ServiceGuideController;
import com.nokia.example.engine.SubscriptionController;

// EXTERNAL IMPORTS
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.esg.ServiceGuideListener;
import javax.microedition.broadcast.esg.ServiceGuideData;
import javax.microedition.broadcast.esg.ProgramEvent;
import javax.microedition.broadcast.esg.CommonMetadataSet;
import javax.microedition.broadcast.esg.DataUnavailableException;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Shows information about program events
 *
 * Events can be either from currently selected service
 * or from a query specified by the user
 */
public class ProgramEventsView extends List
        implements ViewBase, ServiceGuideListener
{
    // Service guide controller
    private ServiceGuideController controller;
    // List of programs
    private Vector programs;
    // Commands
    private Command cmdBack;
    private Command cmdInfo;
    private Command cmdListPurchaseObjects;

    private Initializer initializer;

    // Subscription controller
    private SubscriptionController subscriptionController;

    /**
     * Constructor
     */
    ProgramEventsView()
    {
        super("Program Events", List.IMPLICIT);

        cmdBack = new Command("Back", Command.BACK, 1);
        cmdInfo = new Command("Show details", Command.SCREEN, 1);
        cmdListPurchaseObjects =
                new Command("Get purchase objects", Command.SCREEN, 1);

        addCommand(cmdBack);
        addCommand(cmdInfo);
        addCommand(cmdListPurchaseObjects);

        controller = ServiceGuideController.getInstance();
        programs = new Vector();

        initializer = new Initializer();
        initializer.start();
    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
        String argEvent,
        ServiceGuideData argServiceGuideData,
        Object argEventData)
    {
        if (argEvent.equalsIgnoreCase(ServiceGuideListener.PROGRAM_DELETED))
        {
            synchronized (this)
            {
                removeProgram((ProgramEvent)argServiceGuideData);
            }
        }
        else if (argEvent.equalsIgnoreCase(
                        ServiceGuideListener.SERVICE_GUIDE_BULK_CHANGED) ||
                 argEvent.equalsIgnoreCase(
                        ServiceGuideListener.NEW_PROGRAM_LISTED))
        {
            requestUpdate();
        }
        else if (argEvent.equalsIgnoreCase(
                        ServiceGuideListener.SERVICE_GUIDE_DELETED))
        {
            synchronized (this)
            {
                programs.removeAllElements();
                deleteAll();
            }
        }
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        subscriptionController = SubscriptionController.getInstance();

        programs.removeAllElements();
        deleteAll();

        requestUpdate();
        controller.addListener(this);
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        controller.removeListener(this);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param argListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener argListener)
    {
        super.setCommandListener(argListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param argCommand The Command to be handled
     * @param argDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command argCommand, Displayable argDisplayable)
    {
        if (argCommand == cmdBack)
        {
            ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
            return;
        }

        int varindex = getSelectedIndex();
        if (varindex < 0)
        {
            return;
        }

        ProgramEvent varProgram = (ProgramEvent)programs.elementAt(varindex);

        if (argCommand == cmdInfo)
        {
            new ServiceGuideDataInfo(getString(varindex), this, varProgram);
        }
        else if (argCommand == cmdListPurchaseObjects)
        {
            String varprogramId;
            try
            {
                varprogramId = (String)varProgram.getStringValue(
                                CommonMetadataSet.PROGRAM_ID);
            }
            catch (DataUnavailableException due)
            {
                due.printStackTrace();
                return;
            }

            subscriptionController.setId(varprogramId);
            ViewHandler.activateView(ViewHandler.PURCHASE_OBJECTS_VIEW_ID);
        }
    }

    private synchronized void requestUpdate()
    {
        // Notify updater if not running
        try
        {
            synchronized (initializer)
            {
                initializer.notify();
            }
        }
        catch (Exception e)
        {
        }
    }

    private synchronized void removeProgram(ProgramEvent argProgram)
    {
        int varIndex = programs.indexOf(argProgram);
        if (varIndex != -1)
        {
            programs.removeElement(argProgram);
            delete(varIndex);
        }
    }

    /**
     * Updates the list of program events
     */
    private synchronized void updatePrograms(Vector argPrograms)
    {
        // Check withc services have been removed
        Enumeration varCurrentPrograms = programs.elements();
        while (varCurrentPrograms.hasMoreElements())
        {
            ProgramEvent varProgram =
                    (ProgramEvent)varCurrentPrograms.nextElement();
            if (!argPrograms.contains(varProgram))
            {
                removeProgram(varProgram);
            }
        }

        // Check that which services are new
        Enumeration varNewPrograms = argPrograms.elements();
        while (varNewPrograms.hasMoreElements())
        {
            ProgramEvent varProgram =
                    (ProgramEvent)varNewPrograms.nextElement();
            if (!programs.contains(varProgram))
            {
                addProgram(varProgram);
            }
        }
    }

    /**
     * Adds new program event to the list
     */
    public synchronized void addProgram(ProgramEvent argProgram)
    {
        programs.addElement(argProgram);
        try
        {
            append(argProgram.getStringValue(
                            CommonMetadataSet.PROGRAM_NAME), null);
        }
        catch (DataUnavailableException due)
        {
            append("(Unnamed)", null);
        }

        int varIndex = programs.size() - 1;
        setFont(varIndex, ViewUtils.getDefaultFont());
    }

    /**
     * Inner class which is used for service list initialization
     */
    private class Initializer extends Thread
    {
        /**
         * Constructor
         *
         * @param aUpdateMode Does a full update if true.
         */
        public Initializer()
        {
        }

        /**
         * Run method for this thread
         */
        public void run()
        {
            while (true)
            {
                try
                {
                    synchronized (this)
                    {
                        this.wait();
                    }
                    // Get selected service
                    Service varService = controller.getService();
                    Vector varProgramEvents = null;

                    if (varService != null)
                    {
                        ProgramEvent[] varPrograms =
                                varService.getProgramEvents();
                        int varCount = varPrograms.length;
                        varProgramEvents = new Vector(varCount);
                        for (int varIndex = 0; varIndex < varCount; varIndex++)
                        {
                            varProgramEvents.addElement(
                                    varPrograms[ varIndex ]);
                        }
                    }
                    else
                    {
                        varProgramEvents = controller.executeQuery();
                    }

                    if (varProgramEvents != null)
                    {
                        updatePrograms(varProgramEvents);
                    }
                }
                catch (Exception e)
                {
                    // Interrupted or service guide invalidated
                    e.printStackTrace();
                }
            }
        }
    }
}

// End of File
